configure.ac: Add some missing checks.
[paraslash.git] / sched.c
diff --git a/sched.c b/sched.c
index 6b893d33c9ccb2b07c65fcb8fd02ac168d7bc6f8..ba43b10cff143960d3741fb6c0f7238b351de865 100644 (file)
--- a/sched.c
+++ b/sched.c
@@ -1,24 +1,15 @@
 /*
- * Copyright (C) 2006 Andre Noll <maan@systemlinux.org>
+ * Copyright (C) 2006-2007 Andre Noll <maan@systemlinux.org>
  *
- *     This program is free software; you can redistribute it and/or modify
- *     it under the terms of the GNU General Public License as published by
- *     the Free Software Foundation; either version 2 of the License, or
- *     (at your option) any later version.
- *
- *     This program is distributed in the hope that it will be useful,
- *     but WITHOUT ANY WARRANTY; without even the implied warranty of
- *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *     GNU General Public License for more details.
- *
- *     You should have received a copy of the GNU General Public License
- *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Licensed under the GPL v2. For licencing details see COPYING.
  */
 
 /** \file sched.c paraslash's scheduling functions */
 
+#include <dirent.h> /* readdir() */
+#include <assert.h>
 #include <sys/time.h>
+
 #include "para.h"
 #include "ipc.h"
 #include "fd.h"
 #include "string.h"
 #include "error.h"
 
-/**
- * The scheduler manages two lists of tasks.  The pre_select list contains
- * pointers to functions that are called before calling select() from the main
- * loop. Similarly, \a post_select_list is a list of function pointers each of
- * which is called after the select call.
- */
-struct list_head pre_select_list, post_select_list;
+static struct list_head pre_select_list, post_select_list;
+static int initialized;
 
 static struct timeval now_struct;
-
 struct timeval *now = &now_struct;
 
 static void sched_preselect(struct sched *s)
@@ -71,7 +56,9 @@ static void sched_post_select(struct sched *s)
 /**
  * the core function for all paraslash programs
  *
* Short and sweet. It updates the global \a now pointer, calls all registered
  * pre_select hooks which may set the timeout and add any file descriptors to
  * the fd sets of \a s.  Next, it calls para_select() and makes the result available
  * to the registered tasks by calling their post_select hook.
@@ -81,9 +68,10 @@ static void sched_post_select(struct sched *s)
  *
  * \sa task, now.
  */
-int sched(struct sched *s)
+int schedule(struct sched *s)
 {
-
+       if (!initialized)
+               return -E_NOT_INITIALIZED;
        gettimeofday(now, NULL);
 again:
        FD_ZERO(&s->rfds);
@@ -102,6 +90,17 @@ again:
        goto again;
 }
 
+/**
+ * initialize the paraslash scheduler
+ */
+static void init_sched(void)
+{
+       PARA_INFO_LOG("%s", "initializing scheduler\n");
+       INIT_LIST_HEAD(&pre_select_list);
+       INIT_LIST_HEAD(&post_select_list);
+       initialized = 1;
+};
+
 /**
  * add a task to the scheduler
  *
@@ -114,14 +113,16 @@ again:
  */
 void register_task(struct task *t)
 {
+       if (!initialized)
+               init_sched();
        PARA_INFO_LOG("registering %s (%p)\n", t->status, t);
        if (t->pre_select) {
                PARA_DEBUG_LOG("pre_select: %p\n", &t->pre_select);
-               list_add(&t->pre_select_node, &pre_select_list);
+               para_list_add(&t->pre_select_node, &pre_select_list);
        }
        if (t->post_select) {
                PARA_DEBUG_LOG("post_select: %p\n", &t->pre_select);
-               list_add(&t->post_select_node, &post_select_list);
+               para_list_add(&t->post_select_node, &post_select_list);
        }
 }
 
@@ -135,6 +136,8 @@ void register_task(struct task *t)
  */
 void unregister_task(struct task *t)
 {
+       if (!initialized)
+               return;
        PARA_INFO_LOG("unregistering %s (%p)\n", t->status, t);
        if (t->pre_select)
                list_del(&t->pre_select_node);
@@ -142,30 +145,24 @@ void unregister_task(struct task *t)
                list_del(&t->post_select_node);
 };
 
-/**
- * initialize the paraslash scheduler
- */
-void init_sched(void)
-{
-       INIT_LIST_HEAD(&pre_select_list);
-       INIT_LIST_HEAD(&post_select_list);
-};
-
 /**
  * unregister all tasks
  *
* This will cause \a sched() to return immediately because both the
  * \a pre_select_list and the \a post_select_list are empty.
  */
 void sched_shutdown(void)
 {
        struct task *t, *tmp;
 
+       if (!initialized)
+               return;
        list_for_each_entry_safe(t, tmp, &pre_select_list, pre_select_node)
                unregister_task(t);
        /* remove tasks which do not have a pre_select hook */
        list_for_each_entry_safe(t, tmp, &post_select_list, post_select_node)
                unregister_task(t);
+       initialized = 0;
 };
 
 /**
@@ -181,6 +178,9 @@ char *get_task_list(void)
 {
        struct task *t, *tmp;
        char *msg = NULL;
+
+       if (!initialized)
+               return NULL;
        list_for_each_entry_safe(t, tmp, &pre_select_list, pre_select_node) {
                char *tmp_msg;
                tmp_msg = make_message("%s%p\tpre\t%s\n", msg? msg : "", t, t->status);
@@ -207,13 +207,16 @@ char *get_task_list(void)
  * Find the task identified by \a id, set the tasks' return value to
  * \p -E_TASK_KILLED and call the event handler of the task.
  *
- * \return Positive on sucess, negative if \a id does not correspond to a
- * registered task.
+ * \return Positive on success, negative on errors (e.g. if \a id does not
+ * correspond to a registered task).
  */
 int kill_task(char *id)
 {
        struct task *t, *tmp;
        char buf[20];
+
+       if (!initialized)
+               return -E_NOT_INITIALIZED;
        list_for_each_entry_safe(t, tmp, &pre_select_list, pre_select_node) {
                sprintf(buf, "%p", t);
                if (strcmp(id, buf))
@@ -71,7 +56,9 @@ static void sched_post_select(struct sched *s)
  * the core function for all paraslash programs
  *
- * Short and sweet. It updates the global \a now pointer, calls all registered
+ * \param s pointer to the scheduler struct
+ *
+ * This function updates the global \a now pointer, calls all registered
  * pre_select hooks which may set the timeout and add any file descriptors to
  * the fd sets of \a s.  Next, it calls para_select() and makes the result available
@@ -142,30 +145,24 @@ void unregister_task(struct task *t)
  * unregister all tasks
  *
- * This will cause \a sched() to return immediately because both the
+ * This will cause \a schedule() to return immediately because both the
  * \a pre_select_list and the \a post_select_list are empty.
  */